box = {}
box.__index = box
allBoxes = {}

-- make a moving function 
-- optimising will also be nice (https://jonathanwhiting.com/tutorial/collision/) this allows games to run better
-- for optimising follow the jonathan whiting guide and check each tile for the type of box, if it is solid, it should function as a full solid, slope function as a slope depending on direction

function box.new_box(x,y,w,h,t)
	local self = setmetatable ({},box)
		self.x = x
		self.y = y
		self.w = w
		self.h = h
		self.t = t
		self.centX = self.x + self.w/2
		self.centY = self.y + self.h/2
		self.inX = false
		self.inY = false
		table.insert (allBoxes, self)
	return self
end

function box.move(self,dirx,diry,speed,slopes,dt)
	sDist = slopes.distance(self)
		-- print (sDist)
	-- x movement right
	-- if (self:closest(self:xax(objects))).t == "solid" then
		xcol = {self:dist(self:closest(self:xax(objects)))} 
	-- end	
	
	if (dirx > 0) then
		self.x = self.x + math.abs(math.min((xcol[1] or math.huge) ,speed*dt))
	elseif (dirx < 0) then
		self.x = self.x - math.abs(math.min((xcol[2] or math.huge) , speed*dt))
	end
	-- y movement
	-- if (self:closest(self:yax(objects))).t == "solid" then
		ycol = {self:dist(self:closest(self:yax(objects)))}
	-- end
	-- upward

	if slopes ~= nil then
		if (diry > 0) then
			self.y = self.y - math.abs(math.min((ycol[3] or math.huge), speed*dt))
		elseif (diry < 0) then
			-- print ("speed: " ..(speed*dt))
			-- print ("dist: " .. (sDist))
			print ("smallest val: "..(math.abs(math.min((ycol[4] or math.huge),math.abs(sDist or math.huge), speed*dt))))
			self.y = self.y + math.abs(math.min((ycol[4] or math.huge),math.abs(sDist or math.huge), speed*dt))
			-- print (math.abs(sDist or math.huge))
		end
	else
		if (diry > 0) then
			self.y = self.y - math.abs(math.min((ycol[3] or math.huge), speed*dt))
		elseif (diry < 0) then
			self.y = self.y + math.abs(math.min((ycol[4] or math.huge), speed*dt))
		end
	end

end



-- to update position of all boxes
function box.updatePos ()
	for i = 1, #allBoxes do
		b = allBoxes [i]
		
		b.centX = b.x + b.w/2
		b.centY = b.y + b.h/2
	end
end

function box.col (self,o)
	local a_top = self.y
	local a_bot = self.y + self.h
	local a_right = self.x + self.w
	local a_left = self.x
	
	local b_top = o.y
	local b_bot = o.y + o.h
	local b_right = o.x + o.w
	local b_left = o.x
	collision_hr = (a_right > b_left and b_right > a_left ) or (a_right < b_left and b_right < a_left ) 
	collision_vr = (a_top > b_bot and a_bot < b_top) or (a_top < b_bot and a_bot > b_top)
	
	collision = collision_hr and collision_vr
	
	return collision
end

function box.dist(self, o,t)
	if o ~= nil then
		local a_top = self.y
		local a_bot = self.y + self.h
		local a_right = self.x + self.w
		local a_left = self.x
		
		local b_top = o.y
		local b_bot = o.y + o.h
		local b_right = o.x + o.w
		local b_left = o.x
		
		

		
		local xdistr
		local xdistl 
		local ydistt 
		local ydistb 
		
		if b_left - a_right >= 0 then
			xdistr = b_left - a_right
			else xdistr = math.huge
		end
		
		if a_left - b_right >= 0 then
			xdistl = a_left - b_right
			else xdistl = math.huge
		end
		
		if  a_top - b_bot >= 0 then
			ydistt =  a_top - b_bot
			else ydistt = math.huge
		end
		
		if b_top - a_bot >= 0 then
			ydistb = b_top - a_bot
			else ydistb = math.huge
		end
		
		return xdistr,xdistl,ydistt,ydistb

	end
end

-- cannot do diagonals pls fix 
function box.yax(self,objects)
	local inYAxis = {}
	for i = 1,#objects do
		local o = objects[i]
		if self ~= o then
			if o.x + o.w > self.x and self.x + self.w > o.x then
				if o.inX ~= true then
					o.inY = true
					inYAxis[#inYAxis + 1] = o
				end
			else 
				o.inY = false
				for j = 1, #inYAxis do
					if inYAxis == o then
						inYAxis[j] = nil
					end
				end
			end
		end
	end
	return inYAxis
end

function box.xax(self,objects)
	local inXAxis = {}
	for i = 1,#objects do
		local o = objects[i]
		if self ~= o then
			if o.y + o.h > self.y and self.y + self.h > o.y then
			if o.inY ~= true then
				o.inX = true
				inXAxis[#inXAxis + 1] = o
			end
			else 
				o.inX = false
				for j = 1, #inXAxis do
					if inXAxis == o then
						inXAxis[j] = nil
					end
				end
			end
		end
	end
	return inXAxis
end

function box.dirX (self,dir,objects)
	local indir = {}
	for i = 1,#objects do
		local o = objects
		if dir ~= 0 then
			if dir < 0 and o.x + o.w < self.x then
				indir[#indir + 1] = o
			end
			if dir > 0 and o.x > self.x + self.w then
				indir[#indir + 1] = o
			end
		else
			for j = 1,#indir do 
				if indir[j] == o then
					indir[j] = nil
				end
			end
		end
	end
	return indir
end

function box.dirY (self,dir,objects)
	local indir = {}
	for i = 1,#objects do
		local o = objects
		if dir ~= 0 then
			if dir > 0 and o.y - o.h < self.y then
				indir[#indir + 1] = o
			end
			if dir < 0 and o.y > self.y + self.h then
				indir[#indir + 1] = o
			end
		else
			for j = 1,#indir do 
				if indir[j] == o then
					indir[j] = nil
				end
			end
		end
	end
	return indir
end

function box.closest (self,objects)
	curDist = math.huge
	for i = 1,#objects do
		local o = objects [i]
		if o.t == "solid" then
			local distance = math.sqrt(math.abs(self.centX - o.centX)^2 + math.abs(self.centY - o.centY)^2)
			if distance < curDist then
				curDist = distance
				nearObj = o
			end
		end
		if i == #objects then
			return nearObj
		end
	end
end

function box.closestT (self,objects)
	curDist = math.huge
	for i = 1,#objects do
		local o = objects [i]
		--test do not implement
		love.graphics.print("player x: " .. self.centX .. " y: " .. self.centY,16,16)
		love.graphics.print ("object " .. i .. " x: " .. o.centX .. " y: " .. o.centY,16, 32 + 16*i)
		-------
		
		distX = math.abs(self.centX - o.centX)
		distY = math.abs(self.centY - o.centY)
		local distance = math.sqrt(distX^2 + distY^2)
		if distance < curDist then
			curDist = distance
			nearObj = o
		end
		if i == #objects then
			-- also, does not consider for objects underneath the most upper object
			
			--test do not implement
			love.graphics.print ("x distance : " .. distX .. " y distance : " .. distY , 16 , 32)
			print ("found x:" .. nearObj.centX .. " y: ".. nearObj.centY)	 
			love.graphics.rectangle("fill",nearObj.x, nearObj.y,16,16)
		end
	end
end

function box.showtrue(k)

	if k == "x" then
		for i = 1,#allBoxes do 
			local o = allBoxes[i]
			if o.inX == true then
				love.graphics.setColor(1,0,0)
				love.graphics.rectangle ("fill",o.x,o.y,16,16)
			end
		end
	end
	
	if k == "y" then
		for i = 1,#allBoxes do 
			local o = allBoxes[i]
			if o.inY == true then
				love.graphics.setColor(0,0,1)
				love.graphics.rectangle ("fill",o.x,o.y,16,16)
			end
		end
	end

end

return box
